#
# -----------------------------------------------------------------------
#      program Solar  - final project cscie 236, May 25th 2005
# -----------------------------------------------------------------------
#      Solar Visualization by Russell Lowke
#      from a physical model of planetary formation by John Lowke
#
#  traces path of ejected "planet" from surface of sun under gravity,
#  electrostatic and magnetic forces.
#

# visualization controls
set p_dist     -1                                   ;# distance at which to draw plot graph
set ctrSun     1                                    ;# if true then sun always drawn at center of screen
set drwPoles   1                                    ;# if true then markers are draw for the north and south poles
set pt_shading 1                                    ;# if true then each plot is shaded according to time t
set max 1.2                                            ;# max modifier for pt_shading
set min 0.3                                            ;# min modifier for pt_shading
set magnify    3                                    ;# factor to magnify planet plots so as they may be seen
set n_rake_pts 16                                   ;# number of rake points, including first (START) and last (END)

# general variables
set rsun       0.7E9                                ;# radius sun(m)
set rplanet    6400.0E3                             ;# radius planet(m)
set rmplanet   6.0E24                               ;# mass planet(kg)
set rmsun      2.0E30                               ;# mass sun(kg)
set dt         0.0001                               ;# timestep(days)
set g          6.67e-11                             ;# g is gravitational constant; N m2 kg-2
set pie        3.14159265359                        ;# [expr 2*asin(1.0)]
set epsilon    8.85e-12                             ;# epsilon electrostatic constant; C V-1 m-1
set H          [expr $dt*86400]                     ;# stepsize, dt is in days, hence factor 86400 to bring to seconds
set nsteps     -1                                   ;# maximum number of steps Hamming takes before stopping
set HMAX       [expr 15*$H]                         ;# maximum time
set HMIN       [expr $H/10000]                      ;# minimum time
set CREL       0.001                                ;# relative error that will be tolerated
set CABS       1.0E9                                ;# absolute error
set CFACT      5                                    ;# from hamming

# START plot constants 
set s_bsun       0.16E23                                ;# bsun is magnetic field at surface of sun at sun's equator   (0.1911E23 !)
set s_sunpot   1000.0                                ;# sun potential(V)
set s_velx     [expr 60.0E4 / 1.414]                ;# x velocity of planet(m/s)
set s_vely     0.0                                    ;# y velocity of planet(m/s)
set s_velz     [expr 60.0E4 / 1.414]                ;# z velocity of planet(m/s)
set s_sx       [expr $rsun / 1.414]                    ;# initial position x -- ejection initially in x direction
set s_sy       0.0                                    ;# initial position y
set s_sz       [expr $rsun / 1.414]                    ;# initial position z
set s_colRed   1.0                                    ;# color of plot Red
set s_colGreen 0.3882                                ;# color of plot Green
set s_colBlue  0.2784                                ;# color of plot Blue

# END plot constants
set e_bsun     0.3211E23                            ;# bsun is magnetic field at surface of sun at sun's equator
set e_sunpot   1000.0                                ;# sun potential(V)
set e_velx     [expr 60.0E4 / 1.414]                ;# x velocity of planet(m/s)
set e_vely     0.0                                    ;# y velocity of planet(m/s)
set e_velz     [expr 60.0E4 / 1.414]                ;# z velocity of planet(m/s)
set e_sx       [expr $rsun / 1.414]                    ;# initial position x -- ejection initially in x direction
set e_sy       0.0                                    ;# initial position y
set e_sz       [expr $rsun / 1.414]                    ;# initial position z
set e_colRed   0.2                                    ;# color of plot Red
set e_colGreen 0.63                                    ;# color of plot Green
set e_colBlue  0.79                                    ;# color of plot Blue


package require vtk
package require vtkinteraction

vtkRenderer render
 render SetBackground 0.0 0.0 0.0
 
# sun vtk
vtkSphereSource sunSphere
 sunSphere SetPhiResolution 8
 sunSphere SetThetaResolution 20
 sunSphere SetRadius $rsun
vtkPolyDataMapper sunMapper
 sunMapper SetInput [sunSphere GetOutput]
vtkProperty sunProperties
 sunProperties SetColor 0.952 0.878 0.047
 sunProperties SetDiffuse 0.7
 sunProperties SetSpecular 0.4
 sunProperties SetSpecularPower 20
vtkActor sun
 sun SetMapper sunMapper
 sun SetProperty sunProperties
 sun SetPosition 0 0 0 
render AddActor sun

# planet position vtk
vtkSphereSource sphere
 sphere SetPhiResolution 6
 sphere SetThetaResolution 6
 sphere SetRadius [expr $rplanet * $magnify]
vtkPolyDataMapper sphereMapper
 sphereMapper SetInput [sphere GetOutput] 
    

# --------------------------------------------------------------------------
#  Calculated planetary position and velocity after a timestep of 
#  planetary motion.  Variables are printed and plotted here.
# --------------------------------------------------------------------------
proc output { XX Y DY IC} {
    global n draw_n nsteps sx sy sz s rsun nprint ndprint H velx vely velz 
    global gacc eacc rmaccx rmaccy colRed colGreen colBlue largest
    global render sphereMapper draw_n p_dist
    
    incr n
    incr draw_n
    
    if {$n == $nprint} {
        
        # puts "n = $n t = $XX step = $H sx = $sx sy = $sy sz = $sz 
        # velx = $velx vely = $vely velz = $velz gacc = $gacc eacc = $eacc 
        # rmaccx = $rmaccx rmaccy = $rmaccy"
        puts "$n) step: $H  pos: ($sx, $sy, $sz)"

        set nprint [expr $nprint + $ndprint]
        
        # vtk stuff
    
        vtkProperty propertyPPos$draw_n
          propertyPPos$draw_n SetColor $colRed $colGreen $colBlue
 
        vtkActor planetPosn$draw_n
         planetPosn$draw_n SetMapper sphereMapper
         planetPosn$draw_n SetProperty propertyPPos$draw_n
         planetPosn$draw_n SetPosition $sx $sy $sz 

        render AddActor planetPosn$draw_n
        
        # puts "-> draw_n: $draw_n"
        # save the largest value
        if {[expr abs($sx)] > $largest} {set largest [expr abs($sx)]}
        if {[expr abs($sy)] > $largest} {set largest [expr abs($sy)]}
        if {[expr abs($sz)] > $largest} {set largest [expr abs($sz)]}
        
    }
     
    if {$s < $rsun} { set IC 1 }						;# has fallen into sun
    if {$nsteps != -1 && $n >= $nsteps} { set IC 1 }    ;# has exceeded maximum steps
    
    return $IC
}


# --------------------------------------------------------------------------
#      subroutine dipole(x,y,z,bx,by,bz)
# calculates magnetic fields bx,by,bz at any position x,y,z with the sun 
# at the origin assuming a dipole representation of the mnagnetic field such
# that the field at the surface of the sun at the sun's equatior is bsun.
# --------------------------------------------------------------------------
proc dipole { x y z } {
    global rsun bsun
    
    # dipole position at 0,0,zd and 0,0,-zd at 2/3 of sun's radius
    set zd [expr 2*$rsun/3]
    # set rmag bsun*(rsun**2 + zd**2)**1.5/(2*zd)
    set r1squared [expr pow($x, 2) + pow($y, 2) + pow(($z-$zd), 2)]
    set r1 [expr sqrt($r1squared)]
    set r2squared [expr pow($x, 2) + pow($y, 2) + pow(($z+$zd), 2)]
    set r2 [expr sqrt($r2squared)]
    # set rxy sqrt(pow(x, 2) + pow(y, 2));

    set b1 [expr ($bsun/$r1squared)* pow((pow($rsun,2) + pow($zd,2)),1.5)/(2*$zd)]
    set b2 [expr -($bsun/$r2squared)* pow((pow($rsun,2) + pow($zd,2)),1.5)/(2*$zd)]

    #      rmag = bsun*(rsun**2 + zd**2)**1.5/(2*zd)
    #      b1 = rmag/r1squared
    #      b2 = -rmag/r2squared
    # b1x,b1y,b1z are components of first dipole force in x, y and z direction.
    set b1x [expr $b1*$x/$r1]
    set b1y [expr $b1*$y/$r1]
    set b1z [expr $b1*($z-$zd)/$r1]
    set b2x [expr $b2*$x/$r2]
    set b2y [expr $b2*$y/$r2]
    set b2z [expr $b2*($z+$zd)/$r2]
    set bx  [expr $b1x + $b2x]
    set by  [expr $b1y + $b2y]
    set bz  [expr $b1z + $b2z]
                          
    return [split "0 $bx $by $bz"]
}


# --------------------------------------------------------------------------
#  evaluates velocities and accelerations of planet for any position
# --------------------------------------------------------------------------
proc Funct { XX Y } {
    global g epsilon pie qsun qplanet rmplanet rmsun t sx sy sz velx velz 
    global gacc eacc rmaccx rmaccy rmaccz H s
    
    # s is distance from center of sun
    # sxy is distance projected on xy plane
    # accelleration due to gravity; gacc
    set sx   [lindex $Y 1]
    set velx [lindex $Y 2]
    set sy   [lindex $Y 3]
    set vely [lindex $Y 4]
    set sz   [lindex $Y 5]
    set velz [lindex $Y 6]
    
    set s [expr sqrt(pow($sx,2) + pow($sy,2) + pow($sz,2))]
    set sxy [expr sqrt(pow($sx, 2) + pow($sy, 2))]
    set gacc [expr -$g*$rmsun/pow($s,2)]
    set gaccx [expr $gacc*$sx/$sxy]
    set gaccy [expr $gacc*$sy/$sxy]
    set gaccz [expr $gacc*$sz/$s]
    
    # accelleration due to electrostatic force; eacc
    set eacc [expr $qsun*$qplanet/(4*$pie*$epsilon*$rmplanet*pow($s,2))]
    set eaccx [expr $eacc*$sx/$sxy]
    set eaccy [expr $eacc*$sy/$sxy]
    set eaccz [expr $eacc*$sz/$s]
    
    # accelleration due to magnetic field; rmacc
    set mVector [dipole $sx $sy $sz]
    set bx [lindex $mVector 1]
    set by [lindex $mVector 2]
    set bz [lindex $mVector 3]
    
    set rjx [expr $qplanet*$velx]
    set rjy [expr $qplanet*$vely]
    set rjz [expr $qplanet*$velz]

    set rmaccx [expr ($rjy*$bz - $rjz*$by)/$rmplanet]
    set rmaccy [expr -($rjx*$bz - $rjz*$bx)/$rmplanet]
    set rmaccz [expr ($rjx*$by - $rjy*$bx)/$rmplanet]
    
    # add accelleration components
    set accx [expr $gaccx + $eaccx + $rmaccx]
    set accy [expr $gaccy + $eaccy + $rmaccy]
    set accz [expr $gaccz + $eaccz + $rmaccz]
    # Y(1) = sx, Y(2) = velx, Y(3) = sy, Y(4) = vely, Y(5) = sz, Y(6) = velz
    # DY(1) = velx, DY(2) = -accx, DY(3) = vely, DY(4) = -accy, DY(5) = velz, DY(6) = -accz
    
    return [split "0 [lindex $Y 2] $accx [lindex $Y 4] $accy [lindex $Y 6] $accz"]
}

# --------------------------------------------------------------------------
#  Terminates output if step size is insufficiently small 
#  for relative error tolerance
# --------------------------------------------------------------------------
proc fail { XX Y DY } {
    global H
    
    puts "  FAILD AT  "
    puts "      Time = $XX"
    puts "         Y = $Y"
    puts "        DY = $DY"
    puts " step size = $H"
}

# --------------------------------------------------------------------------
#  Hamming was translated from Fortran 77 by R Lowke
#	"Hamming" is a "predictor corrector" numerical method which 
#   automatically adjusts the time step so that the results have
#   less then a specified relative error, thus timesteps are small
#   near the sun where forces are large and time steps are large
#   away from the sun where forces are small
# --------------------------------------------------------------------------
# HAMMING METHOD FOR SOLVING A SYSTEM OF FIRST ORDER ORDINARY
#     DIFFERENTIAL EQUATIONS
#     K  NUMBER OF EQUATIONS
#     X  INDEPENDENT VARIABLE CONTAINING ITS STARTING VALUE
#     H  VARIABLE CONTAINING STARTING VALUE OF STEP SIZE
#     HMAX MAXIMUM ALLOWABLE STEP SIZE
#     HMIN  MINIMUM ALLOWABLE STEP SIZE
#     CREL  RELATIVE ERROR TOLERANCE
#     CABS  ABSOLUTE ERROR TOLERANCE
#     CFACT  FACTOR USED FOR THE CRITERION FOR DOUBLING H
#     YO ARRAY OF INITIAL VALUES OF DEPENDENCY Y VARIABLES
#     BOXA PROCEDURE FOR EVALUATING THE DERIVATIVES 
#     BOXB PROCEDURE FOR OUTPUT AFTER EACH SUCCESSFUL INTEGRATION   
#     BOXD  PROCEDURE FOR EXITING WHEN ERROR CRITERIA FAILS
#

proc hamming { K X HMAX HMIN CREL CABS ICFACT YCs } {
    global H
    
    # convert YC list to YC array vars...
    set YC(1) [lindex $YCs 1]
    set YC(2) [lindex $YCs 2]
    set YC(3) [lindex $YCs 3]
    set YC(4) [lindex $YCs 4]
    set YC(5) [lindex $YCs 5]
    set YC(6) [lindex $YCs 6]
    
    # assign end items of DTABS to 0
    set DTABS(4) 0
    set DTABS(5) 0
    set DTABS(6) 0
    
    set IC 0
    set RTEST [expr 15*$CREL]
    set ATEST [expr 15*$CABS]
    
    for {set J 1} { $J <= $K } { incr J } {
        set Y($J,1) $YC($J)
    }
    
    if {$HMAX < 0} { set H [expr -abs($H)] }
    
    # THE NEXT TWO STATEMENTS ADDED BY RWL FOR OBTAINING INITIAL CONDITIONS 2/5/75
    set DYYs [Funct $X [split "0 $YC(1) $YC(2) $YC(3) $YC(4) $YC(5) $YC(6)"]]
    set DYY(1) [lindex $DYYs 1]; set DYY(2) [lindex $DYYs 2]; set DYY(3) [lindex $DYYs 3]
    set DYY(4) [lindex $DYYs 4]; set DYY(5) [lindex $DYYs 5]; set DYY(6) [lindex $DYYs 6]
    
    if {$IC == 1} { return }
    
    set IC [output $X [split "0 $YC(1) $YC(2) $YC(3) $YC(4) $YC(5) $YC(6)"] [split "0 $DYY(1) $DYY(2) $DYY(3) $DYY(4) $DYY(5) $DYY(6)"] $IC]
    
    if {$IC == 1} { return }
    
    # RUNGE-KUTTA FOURTH ORDER FOR STARTING INTEGRATION
    set H [expr 2*$H]
    set IA 2
    set IB 2

    for {set loop 1} { $loop == 1 } {} {
        for {set I $IA} {$I <= $IB} {incr I} {

            set IM1 [expr $I-1]
            
            for {set J 1} {$J <= $K} {incr J} {
                set YY($J) $Y($J,$IM1)
            }
            
            set DYYs [Funct $X [split "0 $YY(1) $YY(2) $YY(3) $YY(4) $YY(5) $YY(6)"]]
            set DYY(1) [lindex $DYYs 1]; set DYY(2) [lindex $DYYs 2]; set DYY(3) [lindex $DYYs 3]
            set DYY(4) [lindex $DYYs 4]; set DYY(5) [lindex $DYYs 5]; set DYY(6) [lindex $DYYs 6]
    
            if {$IC == 1} { return }
            
            for {set J 1} {$J <= $K} {incr J} {
                
                set T $DYY($J)
                set DY($J,$IM1) $T
                set T [expr $T*$H]
                set RK1($J) $T
                set T [expr $YY($J) + 0.5*$T]
                set YY($J) $T
                set Y($J,$I) $T
            }
            set XTEMP [expr $X + 0.5*$H]
            set DYYs [Funct $X [split "0 $YY(1) $YY(2) $YY(3) $YY(4) $YY(5) $YY(6)"]]
            set DYY(1) [lindex $DYYs 1]; set DYY(2) [lindex $DYYs 2]; set DYY(3) [lindex $DYYs 3]
            set DYY(4) [lindex $DYYs 4]; set DYY(5) [lindex $DYYs 5]; set DYY(6) [lindex $DYYs 6]
    
            if {$IC == 1} { return }
            
            for {set J 1} {$J <= $K} {incr J} {
                set T $DYY($J)
                set DY($J,$I) $T
                set T [expr $T*$H]
                set RK2($J) $T
                set T [expr $Y($J,$IM1) + 0.5*$T]
                set YY($J) $T
                set Y($J,$I) $T
            }
            
            set DYYs [Funct $XTEMP [split "0 $YY(1) $YY(2) $YY(3) $YY(4) $YY(5) $YY(6)"]]
            set DYY(1) [lindex $DYYs 1]; set DYY(2) [lindex $DYYs 2]; set DYY(3) [lindex $DYYs 3]
            set DYY(4) [lindex $DYYs 4]; set DYY(5) [lindex $DYYs 5]; set DYY(6) [lindex $DYYs 6]
            if {$IC == 1} { return }
            
            for {set J 1} {$J <= $K} {incr J} {
                set T $DYY($J)
                set DY($J,$I) $T
                set T [expr $T*$H]
                set RK3($J) $T
                set T [expr $Y($J,$IM1) + $T]
                set YY($J) $T
                set Y($J,$I) $T
            }
            
            set X [expr $X + $H]
            
            set DYYs [Funct $X [split "0 $YY(1) $YY(2) $YY(3) $YY(4) $YY(5) $YY(6)"]]
            set DYY(1) [lindex $DYYs 1]; set DYY(2) [lindex $DYYs 2]; set DYY(3) [lindex $DYYs 3]
            set DYY(4) [lindex $DYYs 4]; set DYY(5) [lindex $DYYs 5]; set DYY(6) [lindex $DYYs 6]
            if {$IC == 1} { return }
            
            for {set J 1} {$J <= $K} {incr J} {
                set Y($J,$I) [expr $Y($J,$IM1) + 0.166666666666*($RK1($J) + 2*($RK2($J) + $RK3($J)) + $H*$DYY($J))]
                set DY($J,$I) $DYY($J)
            }
        }
            
        if { $IB != 2 } { 
            if { $IB != 3 } { 
                for {set J 1} {$J <= $K} {incr J} {
                    set YY($J) $Y($J,4)
                }
                
                set DYYs [Funct $X [split "0 $YY(1) $YY(2) $YY(3) $YY(4) $YY(5) $YY(6)"]]
                set DYY(1) [lindex $DYYs 1]; set DYY(2) [lindex $DYYs 2]; set DYY(3) [lindex $DYYs 3]
                set DYY(4) [lindex $DYYs 4]; set DYY(5) [lindex $DYYs 5]; set DYY(6) [lindex $DYYs 6]
                
                if {$IC == 1} { return }
                
                for {set J 1} {$J <= $K} {incr J} {
                    set DY(J,4) $DYY($J)
                }
                
                # OUTPUT RUNGE-KUTTA RESULTS
                set X [expr $X-4.0*$H]
                for {set I 1} {$I <= 4} {incr I} {
                    set X [expr $X+$H]
                    for {set J 1} {$J <= $K} {incr J} {
                        set YY($J) $Y($J,$I)
                        set DYY($J) $DY($J,$I)
                    }
                }
                
                set IC [output $X [split "0 $YY(1) $YY(2) $YY(3) $YY(4) $YY(5) $YY(6)"] [split "0 $DYY(1) $DYY(2) $DYY(3) $DYY(4) $DYY(5) $DYY(6)"] $IC]
                if {$IC == 1} { return }
                
                # CONSTANTS AND TEST VARIABLES FOR HAMMING
                set CON2 [expr 1.0/121.0]
                # CON1 = 112.0*CON2
                set CON2 [expr 9.0*$CON2]
                set CON3 [expr 1.0/128.0]
                set T [expr 1.0/$CON2]
                set TREL [expr $CREL*$T]
                set TABS [expr $CABS*$T]
                set DTABS(1) [expr $TABS*1.0E-4]
                set DTABS(2) [expr $TABS*1.0E-5]
                set DTABS(3) [expr $TABS*1.0E-2]
                set T $ICFACT
                
                
                if {$T == 0.0} { set T 100.0 }
                set TDH [expr $TREL/$T]
                set L 1
     
                 for {set loop2 1} { $loop2 == 1 } {} {
                    set gotoL510 0
                
                    # INTEGRATE
                    set X [expr $X+$H]
                    set LP1 [expr $L+1]
                    set LP2 [expr $L+2]
                    set LP3 [expr $L+3]
                    set LP4 [expr $L+4]
                    
                    for {set I 1} {$I <= $K} {incr I} {
                        set C($I) [expr $Y($I,$L) + 1.333333*$H*(2*($DY($I,$LP1) + $DY($I,$LP3)) - $DY($I,$LP2))]
                    }

                    set DCs [Funct $X [split "0 $C(1) $C(2) $C(3) $C(4) $C(5) $C(6)"]]
                    set DC(1) [lindex $DCs 1]; set DC(2) [lindex $DCs 2]; set DC(3) [lindex $DCs 3]
                    set DC(4) [lindex $DCs 4]; set DC(5) [lindex $DCs 5]; set DC(6) [lindex $DCs 6]
                    if {$IC == 1} { return }
                     
                    for {set I 1} {$I <= $K} {incr I} {
                        
                        set T [expr 0.125*(9*$Y($I,$LP3)-$Y($I,$LP1)+3*$H*($DC($I)+2*$DY($I,$LP3)- $DY($I,$LP2)))]
                        set C($I) [expr $C($I) - $T]
                        set Y($I,$LP4) $T
                        set TT [expr $TREL*abs($T)]
                        set TEST [expr abs($C($I))]
                        set TABS $DTABS($I)
                        
                        if {($TEST > $TT) && ($TEST > $TABS)} { 
                        
                            # CHECK FOR HALVING STEP SIZE. IF LESS THAN HMIN, TERMINATE
                            if {[expr abs([expr 0.5*$H])] > [expr abs($HMIN)]} {

                                for {set J 1} {$J <= $K} {incr J} {
                                    
                                    set Y1 $Y($J,$LP1)
                                    set Y2 $Y($J,$LP2)
                                    set Y3 $Y($J,$LP3)
                                    set DY1 $DY($J,$LP1)
                                    set DY2 $DY($J,$LP2)
                                    set DY3 $DY($J,$LP3)
                                    set Y($J,1) [expr $CON3*(11*$Y3+72*$Y2+45*$Y1-$H*(3*$DY3+36*$DY2-9*$DY1))]
                                    set Y($J,2) $Y2
                                    set DY($J,2) $DY2
                                    set Y($J,3) [expr $CON3*(45*$Y3+72*$Y2+11*$Y1+$H*(-9*$DY3+36*$DY2+3.0*$DY1))]
                                    set Y($J,4) $Y3
                                    set DY($J,4) $DY3
                                }
                                
                                set X [expr $X-$H]
                                set H [expr $H/2.0]
                                set L 1
                                set T [expr $X-3.0*$H]
                                set TT [expr $X-$H]
                                
                                for {set I 1} {$I <= $K} {incr I} {
                                    set C($I) $Y($I,1)
                                    set YY($I) $Y($I,3)
                                }
                             
                                set DCs [Funct $T [split "0 $C(1) $C(2) $C(3) $C(4) $C(5) $C(6)"]]
                                set DC(1) [lindex $DCs 1]; set DC(2) [lindex $DCs 2]; set DC(3) [lindex $DCs 3]
                                set DC(4) [lindex $DCs 4]; set DC(5) [lindex $DCs 5]; set DC(6) [lindex $DCs 6]
                    
                                if {$IC == 1} { return }
                                
                                set DYYs [Funct $TT [split "0 $YY(1) $YY(2) $YY(3) $YY(4) $YY(5) $YY(6)"]]
                                set DYY(1) [lindex $DYYs 1]; set DYY(2) [lindex $DYYs 2]; set DYY(3) [lindex $DYYs 3]
                                set DYY(4) [lindex $DYYs 4]; set DYY(5) [lindex $DYYs 5]; set DYY(6) [lindex $DYYs 6]
                                if {$IC == 1} { return }
                                
                                for {set I 1} {$I <= $K} {incr I} {
                                    set DY($I,1) $DC($I)
                                    set DY($I,3) $DYY($I)
                                }
                                
                                set gotoL510 1; break;
                            }
                            
                            for {set I 1} {$I <= $K} {incr I} {
                                set YY($I) $Y($I,$LP3)
                                set DYY($I) $DY($I,$LP3)
                            }

                            set X [expr $X-$H]
                            
                            fail $X $YY
                            return;
                        }
                    }
                    
                    if {$gotoL510 == 1} { continue }
                      
                    for {set I 1} {$I <= $K} {incr I} {
                        set YY($I) [expr $Y($I,$LP4) + $CON2*$C($I)]
                        set Y($I,$LP4) $YY($I)
                    }
                    
                    set DYYs [Funct $X [split "0 $YY(1) $YY(2) $YY(3) $YY(4) $YY(5) $YY(6)"]]
                    set DYY(1) [lindex $DYYs 1]; set DYY(2) [lindex $DYYs 2]; set DYY(3) [lindex $DYYs 3]
                    set DYY(4) [lindex $DYYs 4]; set DYY(5) [lindex $DYYs 5]; set DYY(6) [lindex $DYYs 6]
                    if {$IC == 1} { return }
                      
                    set IC [output $X [split "0 $YY(1) $YY(2) $YY(3) $YY(4) $YY(5) $YY(6)"] [split "0 $DYY(1) $DYY(2) $DYY(3) $DYY(4) $DYY(5) $DYY(6)"] $IC]
                    if {$IC == 1} { return }
                      
                    for {set I 1} {$I <= $K} {incr I} {
                        set DY($I,$LP4) $DYY($I)
                    }
                       
                    # CHECK FOR DOUBLING INTEGRATION STEP
                    if {[expr abs([expr 2.0*$H])] > [expr abs($HMAX)]} { 
                        
                        # RESTORE Y AND DY
                        for {set I 1} {$I <= 4} {incr I} {
                            set LPI [expr $L+$I]
                            for {set J 1} {$J <= $K} {incr J} {
                                set Y($J,$I) $Y($J,$LPI)
                                set DY($J,$I) $DY($J,$LPI)
                            }
                        }
                        set L 1
                        
                        continue ;# gotoL510 
                    }
                    
                    for {set I 1} {$I <= $K} {incr I} {
                        set TT [expr $TDH*abs($YY($I))]
                        set TEST [expr abs($C($I))]
                        set TABS $DTABS($I)
                        if {($TEST > $TT) && ($TEST > $TABS)} { 
                            
                            # RESTORE Y AND DY
                            for {set I 1} {$I <= 4} {incr I} {
                                set LPI [expr $L+$I]
                                for {set J 1} {$J <= $K} {incr J} {
                                    set Y($J,$I) $Y($J,$LPI)
                                    set DY($J,$I) $DY($J,$LPI)
                                }
                            }
                            set L 1
                        
                            set gotoL510 1; break
                        }
                    }
                    if {$gotoL510 == 1} { continue }
                    
                    if { $L != 3 } { 
                        set L $LP1
                    } else {
                    
                        for {set I 1} {$I <= 4} {incr I} {
                            set I2M1 [expr 2*$I-1]
                            for {set J 1} {$J <= $K} {incr J} {
                                set Y($J,$I) $Y($J,$I2M1)
                                set DY($J,$I) $DY($J,$I2M1)
                            }
                        }
                        
                        set H [expr 2*$H]
                        set L 1
                    }
                }
            }
            
            set flag 0
            for {set J 1} {$J <= $K} {incr J} {
            
                set T $Y($J,3)
                set TEST [expr abs($C($J)-$T)]
                set TT [expr abs($T)*$RTEST]
                
                if {(($TEST > $TT) && ($TEST > $ATEST))} {
                    set X [expr $X-$H]
                    set flag 1
                }
            }
            
            if {$flag == 0} {
                set IA 4
                set IB 4
                continue; #gotoL100;
            }
        }
        
        for {set J 1} {$J <= $K} {incr J} {
            set C($J) $Y($J,2)
        }
        
        set X [expr $X-$H]
        set H [expr $H/2.0]
        
        if {[expr abs($H)] >= [expr abs($HMIN)]} {
            set IB 3
            continue ; # gotoL100
        }
        
        for {set J 1} {$J <= $K} {incr J} {
            set YY($J) $Y($J,1)
            set DYY($J) $DY($J,1)
        }

        fail $X $YY
        return
    }
}

# draws edge plots so the sun will be center screen
# also draws north and south pole dots
proc drawEdge { edge_at } {
    global ctrSun drwPoles rsun renWin
    
    if {$ctrSun == 1} {
    
        # draw "edges" so to place the sun always in the middle of screen
        vtkProperty edgeProperties
         edgeProperties SetColor 0 0 0
        vtkActor edge_a
         edge_a SetMapper sphereMapper
         edge_a SetProperty edgeProperties
         edge_a SetPosition $edge_at 0 0 
        render AddActor edge_a
        vtkActor edge_b
         edge_b SetMapper sphereMapper
         edge_b SetProperty edgeProperties
         edge_b SetPosition -$edge_at 0 0
        render AddActor edge_b
        vtkActor edge_c
         edge_c SetMapper sphereMapper
         edge_c SetProperty edgeProperties
         edge_c SetPosition 0 $edge_at 0 
        render AddActor edge_c
        vtkActor edge_d
         edge_d SetMapper sphereMapper
         edge_d SetProperty edgeProperties
         edge_d SetPosition 0 -$edge_at 0
        render AddActor edge_d
    
    }
    
    if {$drwPoles == 1} {
        vtkSphereSource pole
         pole SetPhiResolution 8
         pole SetThetaResolution 8
         pole SetRadius 51200000.0
        vtkPolyDataMapper poleMapper
         poleMapper SetInput [pole GetOutput] 
        vtkProperty north
         north SetColor 0.9 0 0                      ;# red north pointer
        vtkActor north_ptr
         north_ptr SetMapper poleMapper
         north_ptr SetProperty north
         north_ptr SetPosition 0 0 [expr $rsun*4]
        render AddActor north_ptr
        vtkProperty south
         south SetColor 0 0 0.9                      ;# blue south pointer
        vtkActor south_ptr
         south_ptr SetMapper poleMapper
         south_ptr SetProperty south
         south_ptr SetPosition 0 0 [expr -$rsun*4]
        render AddActor south_ptr
    }
}

vtkRenderWindow renWin
 renWin AddRenderer render
 renWin SetSize 500 500
 
if {$ctrSun == 0 && $p_dist > 0} {set p_dist -1; puts "Note: p_dist ignored as ctrSun = false"}
if {$p_dist > 0 } { drawEdge $p_dist; renWin Render }

# print vars to screen
puts "Program Solar"

set largest 0                                        ;# var for saving the largest location value
set draw_n 0

for {set i 0} {$i < $n_rake_pts} {incr i} {

    puts ""
    puts "Rake: [expr $i+1]"
    
    set t 0.0                                        ;# time
    set n 0                                          ;# step number
    set ndprint    1                                 ;# prints every nd'th step
    set nprint     $ndprint                          ;# same as ndprint, used for hamming
    
    set rake_pts $n_rake_pts
    if {$n_rake_pts == 1} {set rake_pts 2}           ;# avoid divide by zero error when using only one rake pt
    
    # assign plot variables from start and end rake points
    set bsun     [expr $s_bsun     + $i * (($e_bsun - $s_bsun)/($rake_pts-1))]
    set sunpot   [expr $s_sunpot   + $i * (($e_sunpot - $s_sunpot)/($rake_pts-1))]
    set velx     [expr $s_velx     + $i * (($e_velx - $s_velx)/($rake_pts-1))]
    set vely     [expr $s_vely     + $i * (($e_vely - $s_vely)/($rake_pts-1))]
    set velz     [expr $s_velz     + $i * (($e_velz - $s_velz)/($rake_pts-1))]
    set sx       [expr $s_sx       + $i * (($e_sx - $s_sx)/($rake_pts-1))]
    set sy       [expr $s_sy       + $i * (($e_sy - $s_sy)/($rake_pts-1))]
    set sz       [expr $s_sz       + $i * (($e_sz - $s_sz)/($rake_pts-1))]
    set colRed   [expr $s_colRed   + $i * (($e_colRed - $s_colRed)/($rake_pts-1))]
    set colGreen [expr $s_colGreen + $i * (($e_colGreen - $s_colGreen)/($rake_pts-1))]
    set colBlue  [expr $s_colBlue  + $i * (($e_colBlue - $s_colBlue)/($rake_pts-1))]
    
    set qsun [expr 4*$pie*$sunpot*$epsilon*$rsun]                  ;# charge on the sun
    set qplanet [expr 4*$pie*(2*$sunpot/3)*$epsilon*$rplanet]      ;# charge on the planet
    set Y [split "0 $sx $velx $sy $vely $sz $velz"]                ;# array of vars for hamming

    puts "Radius Sun = $rsun m,  radius planet = $rplanet m, mass sun = $rmsun kg, mass planet = $rmplanet kg"
    puts "Potential Sun = $sunpot V  Sun magnetic B field $bsun T"
    puts "q Sun = $qsun C  q Planet = $qplanet C"
    puts "Y = $Y"
    puts ""
    hamming 6 $t $HMAX $HMIN $CREL $CABS $CFACT $Y
    
    # handle shading of rake plot
    if {$pt_shading == 1} {
        set shade_n [expr $draw_n - $n]
        set range [expr $max - $min]
        set range_inc [expr $range / $n]
        
        for {set j 0} {$j < $n} {incr j} {
            incr shade_n
            
            set modifier [expr $max - $range_inc*$j]
            propertyPPos$shade_n SetColor [expr $colRed * $modifier] [expr $colGreen * $modifier] [expr $colBlue * $modifier]
        }
    }
}

if {$p_dist <= 0 } { drawEdge $largest; puts "Largest distance: $largest" }
renWin Render

vtkRenderWindowInteractor iren
 iren SetRenderWindow renWin

vtkInteractorStyleTrackballCamera style
 iren SetInteractorStyle style
 iren AddObserver UserEvent {wm deiconify .vtkInteract}
 iren Initialize

puts "FINISHED"

wm withdraw .