//-----------------------------------------------------------------------
// program Solar  - final project cscie 236, May 25th 2005
//-----------------------------------------------------------------------
//
// traces path of ejected "planet" from surface of sun under gravity,
// electrostatic and magnetic forces.
//
// Translated from a Fortran 77 code of John Lowke
// 
// translated to C++ (and goto statements removed) by Russell Lowke
//  

#include "Matrix.h"

#include <cmath>

// #include <cstdlib>
// #include <iostream>

using namespace std;

// global vars for John Lowke
double bsun; 
double rsun;
double rplanet;
double g;
double epsilon;
double sunpot;
double pie;
double qsun;
double qplanet;
double rmplanet;
double rmsun;
double t;
int n;
int nsteps;
int ndprint;
int nprint;
double sx;
double sy;
double sz;
double velx;
double vely;
double velz;
double gacc;
double eacc;
double rmaccx;
double rmaccy;
double rmaccz;
double H;
double s;

// --------------------------------------------------------------------------
//      subroutine dipole(x,y,z,bx,by,bz)
// calculates magnetic fields bx,by,bz at any position x,y,z with the sun 
// at the origin assuming a dipole representation of the mnagnetic field such
// that the field at the surface of the sun at the sun's equatior is bsun.
// --------------------------------------------------------------------------
Point3D dipole(double x,
			   double y,
			   double z) {
					  
	// dipole position at 0,0,zd and 0,0,-zd at 2/3 of sun's radius
	double zd = 2*rsun/3;
	//      rmag = bsun*(rsun**2 + zd**2)**1.5/(2*zd)
	double r1squared = pow(x, 2) + pow(y, 2) + pow((z-zd), 2);
	double r1 = sqrt(r1squared);
	double r2squared = pow(x, 2) + pow(y, 2) + pow((z+zd), 2);
	double r2 = sqrt(r2squared);
	// double rxy = sqrt(pow(x, 2) + pow(y, 2));

	double b1 = (bsun/r1squared)* pow(( pow(rsun,2) + pow(zd,2)),1.5)/(2*zd);
	double b2 = -(bsun/r2squared)* pow((pow(rsun,2) + pow(zd,2)),1.5)/(2*zd);

	//      rmag = bsun*(rsun**2 + zd**2)**1.5/(2*zd)
	//      b1 = rmag/r1squared
	//      b2 = -rmag/r2squared
	// b1x,b1y,b1z are components of first dipole force in x, y and z direction.
	double b1x = b1*x/r1;
	double b1y = b1*y/r1;
	double b1z = b1*(z-zd)/r1;
	double b2x = b2*x/r2;
	double b2y = b2*y/r2;
	double b2z = b2*(z+zd)/r2;
	double bx = b1x + b2x;
	double by = b1y + b2y;
	double bz = b1z + b2z;
						  
	return Point3D(bx,by,bz);
}

Array F(double XX, Array Y) {
	
	// s is distance from center of sun
	// sxy is distance projected on xy plane
	// accelleration due to gravity; gacc
	sx = Y(1);
	velx = Y(2);
	sy = Y(3);
	vely = Y(4);
	sz = Y(5);
	velz = Y(6);
	
	s = sqrt(pow(sx,2) + pow(sy,2) + pow(sz,2));
	double sxy = sqrt(pow(sx, 2) + pow(sy, 2));
	gacc = -g*rmsun/pow(s,2);
	double gaccx = gacc*sx/sxy;
	double gaccy = gacc*sy/sxy;
	double gaccz = gacc*sz/s;
	
	// accelleration due to electrostatic force; eacc
	eacc = qsun*qplanet/(4*pie*epsilon*rmplanet*pow(s,2));
	double eaccx = eacc*sx/sxy;
	double eaccy = eacc*sy/sxy;
	double eaccz = eacc*sz/s;
	
	// accelleration due to magnetic field; rmacc
	Point3D mVector = dipole(sx,sy,sz);
	
	double bx = mVector.x();
	double by = mVector.y();
	double bz = mVector.z();
	
	double rjx = qplanet*velx;
	double rjy = qplanet*vely;
	double rjz = qplanet*velz;
	rmaccx = (rjy*bz - rjz*by)/rmplanet;
	rmaccy = -(rjx*bz - rjz*bx)/rmplanet;
	rmaccz = (rjx*by - rjy*bx)/rmplanet;
	// add accelleration components
	double accx = gaccx + eaccx + rmaccx;
	double accy = gaccy + eaccy + rmaccy;
	double accz = gaccz + eaccz + rmaccz;
	// Y(1) = sx, Y(2) = velx, Y(3) = sy, Y(4) = vely, Y(5) = sz, Y(6) = velz
	// DY(1) = velx, DY(2) = -accx, DY(3) = vely, DY(4) = -accy, DY(5) = velz, DY(6) = -accz

	Array DY(6);

	DY.set(1, Y(2));
	DY.set(2, accx);
	DY.set(3, Y(4));
	DY.set(4, accy);
	DY.set(5, Y(6));
	DY.set(6, accz);
	  
	return DY;
}

int output(double XX, Array Y, Array DY, int IC) {

	++n;
	
	if (n == nprint) {
	
		cout << n  << " \t\t ";
		// cout << XX << " \t\t ";
		// cout << H  << " \t\t ";
		cout << sx << " \t\t ";
		cout << sy << " \t\t ";
		cout << sz << endl;
		
		/*
		cout << velx << " \t\t ";
		cout << vely << " \t\t ";
		cout << gacc << " \t\t ";
		cout << eacc << " \t\t ";
		cout << rmaccx << " \t\t ";
		cout << rmaccy << endl;
		*/  

		nprint = nprint + ndprint;
	}
	
	// cout << "before IC = " << IC << endl;
	// cout << "s: " << s << " rsun: " << rsun << endl;
	// cout << "n: " << n << " nsteps: " << nsteps << endl;
	 
	if (s < rsun)    { IC = 1; }		// has fallen into sun
	if (n >= nsteps) { IC = 1; }		// has exceeded maximum steps
	// cout << "after IC = " << IC << endl;
	
	return IC;
}

void fail(double XX, Array Y, Array DY) {
	cout << "  FAILD AT   Time:" << XX << " x position:" << Y(1) << " y position: " << Y(2) << "  step size:" << H << endl;
}


// HAMMING METHOD FOR SOLVING A SYSTEM OF FIRST ORDER ORDINARY
//     DIFFERENTIAL EQUATIONS
//     K  NUMBER OF EQUATIONS
//     X  INDEPENDENT VARIABLE CONTAINING ITS STARTING VALUE
//     H  VARIABLE CONTAINING STARTING VALUE OF STEP SIZE
//     HMAX MAXIMUM ALLOWABLE STEP SIZE
//     HMIN  MINIMUM ALLOWABLE STEP SIZE
//     CREL  RELATIVE ERROR TOLERANCE
//     CABS  ABSOLUTE ERROR TOLERANCE
//     CFACT  FACTOR USED FOR THE CRITERION FOR DOUBLING H
//     YO ARRAY OF INITIAL VALUES OF DEPENDENCY Y VARIABLES
//     BOXA PROCEDURE FOR EVALUATING THE DERIVATIVES 
//     BOXB PROCEDURE FOR OUTPUT AFTER EACH SUCCESSFUL INTEGRATION   
//     BOXD  PROCEDURE FOR EXITING WHEN ERROR CRITERIA FAILS
//
void hamming(int K, double X, double HMAX, double HMIN, double CREL, double CABS, int ICFACT, Array YC) {

	Array2D Y(7,9);
	Array2D DY(7,9);
	Array C(7);
	Array DC(7);
	Array YY(7);
	Array DYY(7);
	Array RK1(7);
	Array RK2(7);
	Array RK3(7);
	Array DTABS(7);
	
	int IA, IB, IM1, L, LP1, LP2, LP3, LP4, I2M1;
	double TEST, T, TT;
	double CON2, CON3, TREL, TABS, TDH;
   
	int IC = 0;
	double RTEST=15*CREL;
	double ATEST=15*CABS;
	
	for (int J = 1; J <= K; J++ ) {
		Y.setAt(J,1, YC(J));
	}
	
	if (HMAX < 0) {
		H = -fabsf(H);
	}
    
	//  THE NEXT TWO STATEMENTS ADDED BY RWL FOR OBTAINING INITIAL CONDITIONS 2/5/75
	DYY = F(X, YC);
	if (IC == 1) { return; }
	
	IC = output(X, YC, DYY, IC);
	if (IC == 1) { return; }
	
	// RUNGE-KUTTA FOURTH ORDER FOR STARTING INTEGRATION
	H = 2*H;
	IA = 2;
	IB = 2;

	for (bool loop = true; loop; ) {
		for (int I = IA; I <= IB; ++I) {

			IM1 = I-1;
			
			for (int J = 1; J <= K; ++J) {
				YY.set(J, Y(J,IM1));
			}
			
			DYY = F(X, YY);
			if (IC == 1) { return; }
			
			double T;
			for (int J = 1; J <= K; ++J) {
				
				T = DYY(J);
				DY.setAt(J,IM1, T);
				T = T*H;
				RK1.set(J, T);
				T = YY(J) + 0.5*T;
				YY.set(J, T);
				Y.setAt(J,I, T);
			}
			double XTEMP = X + 0.5*H;
			DYY = F(X, YY);
			if (IC == 1) { return; }
			
			for (int J = 1; J <= K; ++J) {
				T = DYY(J);
				DY.setAt(J,I, T);
				T = T*H;
				RK2.set(J, T);
				T = Y(J,IM1) + 0.5*T;
				YY.set(J, T);
				Y.setAt(J,I, T);
			}
			
			DYY = F(XTEMP, YY);
			if (IC == 1) { return; }
			
			for (int J = 1; J <= K; ++J) {
				T = DYY(J);
				DY.setAt(J,I, T);
				T = T*H;
				RK3.set(J, T);
				T = Y(J,IM1) + T;
				YY.set(J, T);
				Y.setAt(J,I, T);
			}
			
			X = X + H;
			
			DYY = F(X, YY);
			if (IC == 1) { return; }
			
			for (int J = 1; J <= K; ++J) {
				Y.setAt(J,I, (Y(J,IM1) + 0.166666666666*(RK1(J) + 2*(RK2(J) + RK3(J)) + H*DYY(J))) );
				DY.setAt(J,I, DYY(J));
			}
		}
			
		if ( IB != 2 ) { 
			if ( IB != 3 ) { 
				for (int J = 1; J <= K; ++J) {
					 YY.set(J, Y(J,4));
				}
				
				DYY = F(X, YY);
				
				if (IC == 1) { return; }
				
				for (int J = 1; J <= K; ++J) {
					DY.setAt(J,4, DYY(J));
				}
				
				// OUTPUT RUNGE-KUTTA RESULTS
				X = X-4.0*H;
				for (int I = 1; I <= 4; ++I) {
					X = X+H;
					for (int J = 1; J <= K; ++J) {
						YY.set(J, Y(J,I));
						DYY.set(J, DY(J,I));
					}
				}
				
				IC = output(X, YY, DYY, IC);
				if (IC == 1) { return; }
				
				// CONSTANTS AND TEST VARIABLES FOR HAMMING
				CON2 = 1.0/121.0;
				// double CON1 = 112.0*CON2;
				CON2 = 9.0*CON2;
				CON3 = 1.0/128.0;
				T = 1.0/CON2;
				TREL = CREL*T;
				TABS = CABS*T;
				DTABS.set(1, TABS*1.0E-4);
				DTABS.set(2, TABS*1.0E-5);
				DTABS.set(3, TABS*1.0E-2);
				T = ICFACT;
				
				if (T == 0.0) { T = 100.0; }
				TDH = TREL/T;
				L = 1;
	 
				bool gotoL510;
				for(int nnn = 1; nnn <= nsteps; ++nnn) {
					gotoL510 = false;
				
					// INTEGRATE
					X = X+H;
					LP1 = L+1;
					LP2 = L+2;
					LP3 = L+3;
					LP4 = L+4;
					
					for (int I = 1; I <= K; ++I) {
						C.set(I, (Y(I,L) + 1.333333*H*(2*(DY(I,LP1) + DY(I,LP3)) - DY(I,LP2))) );
					}

					DC = F(X, C);
					if (IC == 1) { return; }
					  
					for (int I = 1; I <= K; ++I) {
						
						T = 0.125*(9*Y(I,LP3)-Y(I,LP1)+3*H*(DC(I)+2*DY(I,LP3)- DY(I,LP2)));
						C.set(I, (C(I) - T));
						Y.setAt(I,LP4, T);
						TT = TREL*fabsf(T);
						TEST = fabsf(C(I));
						TABS = DTABS(I);
						if ((TEST > TT) && (TEST > TABS)) { 
						
							// CHECK FOR HALVING STEP SIZE. IF LESS THAN HMIN, TERMINATE
							if (fabsf(0.5*H) > fabsf(HMIN)) {
								double Y1, Y2, Y3, DY1, DY2, DY3;
								for (int J = 1; J <= K; ++J) {
									Y1 = Y(J,LP1);
									Y2 = Y(J,LP2);
									Y3 = Y(J,LP3);
									DY1 = DY(J,LP1);
									DY2 = DY(J,LP2);
									DY3 = DY(J,LP3);
									Y.setAt(J,1, (CON3*(11*Y3+72*Y2+45*Y1-H*(3*DY3+36*DY2- 9*DY1))));
									Y.setAt(J,2, Y2);
									DY.setAt(J,2, DY2);
									Y.setAt(J,3, (CON3*(45*Y3+72*Y2+11*Y1+H*(-9*DY3+36*DY2+ 3.0*DY1))));
									Y.setAt(J,4, Y3);
									DY.setAt(J,4, DY3);
								}
								
								X = X-H;
								H = H/2.0;
								L = 1;
								T = X-3.0*H;
								TT = X-H;
								  
								for (int I = 1; I <= K; ++I) {
									C.set(I, Y(I,1));
									YY.set(I, Y(I,3));
								}
							 
								DC = F(T, C);		// CALL BOXA(T,C,DC,IC)
								if (IC == 1) { return; }
								
								DYY = F(TT, YY);	// CALL BOXA(TT,YY,DYY,IC)
								if (IC == 1) { return; }
								
								for (int I = 1; I <= K; ++I) {
									DY.setAt(I,1, DC(I));
									DY.setAt(I,3, DYY(I));
								}
								
								gotoL510 = true; break;
							}
							
							for (int I = 1; I <= K; ++I) {
								YY.set(I, Y(I,LP3));
								DYY.set(I, DY(I,LP3));
							}

							X = X-H;
							
							fail(X, YY, DYY);
							return;
						}
					}
					
					if (gotoL510) { continue; }
					  
					for (int I = 1; I <= K; ++I) {
						YY.set(I, (Y(I,LP4) + CON2*C(I)) );
						Y.setAt(I,LP4, YY(I));
					}
					

					DYY = F(X, YY);
					if (IC == 1) { return; }
					  
					IC = output(X, YY, DYY, IC);
					if (IC == 1) { return; }
					  
					for (int I = 1; I <= K; ++I) {
						DY.setAt(I,LP4, DYY(I));
					}
					   
					// CHECK FOR DOUBLING INTEGRATION STEP
					if (fabsf(2.0*H) > fabsf(HMAX)) { 
						
						// RESTORE Y AND DY
						int LPI;
						for (int I = 1; I <= 4; ++I) {
							LPI = L+I;
							for (int J = 1; J <= K; ++J) {
								Y.setAt(J,I, Y(J,LPI));
								DY.setAt(J,I, DY(J,LPI));
							}
						}
						L = 1;
						
						continue; // gotoL510; 
					}
					
					for (int I = 1; I <= K; ++I) {
						TT = TDH*fabsf(YY(I));
						TEST = fabsf(C(I));
						TABS = DTABS(I);
						if ((TEST > TT) && (TEST > TABS)) { 
							
							// RESTORE Y AND DY
							int LPI;
							for (int I = 1; I <= 4; ++I) {
								LPI = L+I;
								for (int J = 1; J <= K; ++J) {
									Y.setAt(J,I, Y(J,LPI));
									DY.setAt(J,I, DY(J,LPI));
								}
							}
							L = 1;
						
							gotoL510 = true; break;
						}
					}
					if (gotoL510) { continue; }
					
					if ( L != 3) { 
						L = LP1;
					} else {
					
						for (int I = 1; I <= 4; ++I) {
							I2M1=2*I-1;
							for (int J = 1; J <= K; ++J) {
								Y.setAt(J,I, Y(J,I2M1));
								DY.setAt(J,I, DY(J,I2M1));
							}
						}
						
						H = 2*H;
						L = 1;
					}
				}
			}
			
			bool flag = false;
			for (int J = 1; J <= K; ++J) {
			
				T = Y(J,3);
				TEST = fabsf(C(J)-T);
				TT = fabsf(T)*RTEST;
				
				if (((TEST > TT) && (TEST > ATEST))) {
					X = X-H;
					flag = true;
				}
			}
			
			if (! flag) {
				IA = 4;
				IB = 4;
				continue; // gotoL100;
			}
		}
		
		for (int J = 1; J <= K; ++J) {
			C.set(J, Y(J,2));
		}
		
		X = X-H;
		H = H/2.0;
		
		if (fabsf(H) >= fabsf(HMIN) ) {
			IB = 3;
			continue; // gotoL100;
		}
		
		for (int J = 1; J <= K; ++J) {
			YY.set(J, Y(J,1));
			DYY.set(J, DY(J,1));
		}

		fail(X, YY, DYY);
		return;
	}
}


// -----------------------------------------------------------------------
//      program  solar
// -----------------------------------------------------------------------
//  traces path of ejected "planet" from surface of sun under gravity,
//  electrostatic and magnetic forces.
//  solar2.f same as solar1.f except that uses HAMMING for predictor-corrector 
//     method to control step size and Runge Kutta.
//
int main(int argc, char* argv[]) {
	
	/*
	bsun = 0.1E23;					// bsun is magnetic field at surface of sun at sun's equator
	rsun = 0.7E9;					// radius sun(m)
	rplanet = 6400.0E3;				// radius planet(m)
	rmplanet = 6.0E24;				// mass planet(kg)
	rmsun = 2.0E30;					// mass sun(kg)
	sunpot = 1000.0;				// sun potential(V);
	double dt = 0.0001;				// timestep(days)
	double velplanet = 60.0E4;		// velplanet(m/s)
	sx = 0.7E9;						// initial position x -- ejection initially in x direction
	sy = 0.0;						// initial position y
	sz = 0.0;						// initial position z
	*/
	
	
	// physical constants
	bsun = 0.15E23;					// bsun is magnetic field at surface of sun at sun's equator
	rsun = 0.7E9;					// radius sun(m)
	rplanet = 6400.0E3;				// radius planet(m)
	rmplanet = 6.0E24;				// mass planet(kg)
	rmsun = 2.0E30;					// mass sun(kg)
	sunpot = 1000.0;				// sun potential(V);
	double dt = 0.0001;				// timestep(days)	
	double velx = 60.0E4;			// velplanet(m/s)
	double vely = 0.0;
	double velz = 0.0;
	sx = 0.7E9;						// initial position x -- ejection initially in x direction
	sy = 0.0;						// initial position y
	sz = 0.0;						// initial position z

	nsteps = 400;
	ndprint = 1;

	// print vars to screen
	cout << "Radius Sun = " << rsun << " m, ";
	cout << "radius planet = " << rplanet << " m, " ;
	cout << "mass sun = " << rmsun << " kg, ";
	cout << "mass planet = " << rmplanet << " kg" << endl;
	
	cout << "Potential Sun = " << sunpot << " V ";
	cout << "Sun magnetic B field " << bsun << " T" << endl;
	
	g = 6.67e-11;								// g is gravitational constant; N m2 kg-2
	pie = 3.14;
	epsilon = 8.85e-12;					// epsilon electrostatic constant; C V-1 m-1
	qsun = 4*pie*sunpot*epsilon*rsun;
	qplanet = 4*pie*(2*sunpot/3)*epsilon*rplanet;
	
	cout << "q Sun = " << qsun << " C ";
	cout << "q Planet = " << qplanet << " C" << endl;
	
	
	// H,HMAX,HMIN are step sizes, CREL,CABS, relative and absolue errors.
	H = dt*86400;		// stepsize, dt is in days, hence factor 86400 to bring to seconds
	double HMAX = 10000*H;		// maximum time
	double HMIN = H/10000;		// minimum time
	double CREL = 0.001;
	double CABS = 1.0E9;		// 1.0E-10
	int CFACT = 5;  
	t = 0.0;

	
	
	Array Y(6);
	Y.set(1, sx);
	Y.set(2, velx);
	Y.set(3, sy);
	Y.set(4, vely);
	Y.set(5, sz);
	Y.set(6, velz);

	nprint = ndprint;
	
	// heading
	cout << "n \t\t ";
	// cout << "t \t\t ";
	// cout << "step size \t\t ";
	cout << "sx(m) \t\t ";
	cout << "sy(m) \t\t ";
	cout << "sz(m) " << endl;
	
	// velx(m/s) \t\t vely(m/s) \t\t gacc \t\t eacc \t\t maccx \t\t maccy" << endl;

	n = 0;
	hamming(6,t,HMAX,HMIN,CREL,CABS,CFACT,Y);

	cout << "FINISHED" << endl;
	for (int i = 0; i == 0;) {}   // loop forever

    return 0;

}