#
# CSCIE-236 Homework 2      
# Russell Lowke  March 22nd 2005
#
# Question 4.1  Create the data set - 15pts
#
# Create a rectilinear Gaussuan volume of 64^3 voxels
#
#  Note: I orignally did this problem using a StructuredGrid data set
#        instead of StructuredPoints.  It turned out later that the
#		 StructuredPoints version was far more flexiable and appropriate.
#		 I have included here the script for both StructuredPoints and 
#		 StructuredGrid, but only save out the StructuredPoints data.
#		 A lot of work went into discovering how to get the StructuredGrid 
#        working. At the end of this script, after saving the StructuredPoints, 
#		 I render the StructuredGrid "for curiosity," being reluctant to lose
#		 the code I had originally worked on.
# 
# ... when convverting C++ to TCL
# anActor->GetProperty()->SetColor(red, green, blue);     is
# [anActor GetProperty] SetColor $red $green $blue

#
# First include the VTK Tcl packages which will make available 
# all of the vtk commands to Tcl
#
package require vtk
package require vtkinteraction
package require vtktesting

# height width and length of cube of data
set cube_size 64

# calc total number of voxels in the cube
set nValues [expr $cube_size*$cube_size*$cube_size]
set pi [expr 2*asin(1.0)]
# location of swarm-- in the middle of cube
set mu_X [expr $cube_size/2]
set mu_Y [expr $cube_size/2]
set mu_Z [expr $cube_size/2]
# size of swarm-- elongated slightly
set sigma_X [expr $cube_size/9]
set sigma_Y [expr $cube_size/7]
set sigma_Z [expr $cube_size/7]

# create structured points data set
vtkStructuredPoints volume
 volume SetDimensions $cube_size $cube_size $cube_size
 volume SetOrigin 0 0 0
 volume SetSpacing 1 1 1
vtkFloatArray scalars
 scalars SetNumberOfValues $nValues
 
# create structured grid, used only as a 
# curiosity to display/check the data after writing
vtkStructuredGrid sgrid
 sgrid SetDimensions $cube_size $cube_size $cube_size
vtkPoints points
vtkFloatArray vectors
 vectors SetNumberOfComponents 3
 vectors SetNumberOfTuples $nValues
 
 # n used as counter to the voxel being processed
set n 0
for {set z 0} {$z < $cube_size} {incr z} {
	for {set y 0} {$y < $cube_size} {incr y} {
		for {set x 0} {$x < $cube_size} {incr x} {
      
			# 3D Gaussian function
			set base [expr 1/(2*$pi*sqrt(2*$pi)*$sigma_X*$sigma_Y*$sigma_Z)]
			set expon [expr -(pow($x-$mu_X,2)/(2*pow($sigma_X,2)) + pow($y-$mu_Y,2)/(2*pow($sigma_Y,2)) + pow($z-$mu_Z,2)/(2*pow($sigma_Z,2)))]
			set val [expr $base*exp($expon)]
      
			# add scalar value to structured points
			scalars InsertValue $n $val
      
			# output boundry values, used later when rendering
			if { $x == $y && $y == $z} {
				puts "value at x:$x y:$y z:$z = $val"
			}
      
			# add points and vectors to structured grid
			# Note: structured grid only implemented for curiosity
			#       as structured points are used for the assignment
			#       I multiply $val by $nValues here to ensure that the
			#       Gaussian will be visible when rendered
			set valB [expr $val*$nValues]
			points InsertPoint $n $x $y $z
			vectors InsertTuple3 $n $valB $valB $valB
      
			incr n
		}
	}
}

# assign scalers to structuredPoints
 [volume GetPointData] SetScalars scalars
 
# assign points and vectors to structured grid
sgrid SetPoints points;
 [sgrid GetPointData] SetVectors vectors

puts "Saving Gaussian data to gaussianDataPts.vtk"
# important part, save out the structured points data
vtkStructuredPointsWriter writer
 writer SetInput volume
 writer SetFileName "gaussianDataPts.vtk"
 writer SetFileTypeToBinary
 writer Write


# for curiosity, display the structured grid

# create a simple pipeline to display the data.
vtkHedgeHog hedgehog
 hedgehog SetInput sgrid
 hedgehog SetScaleFactor 0.1

vtkPolyDataMapper sgridMapper
 sgridMapper SetInput [hedgehog GetOutput]
vtkActor sgridActor
 sgridActor SetMapper sgridMapper
 [sgridActor GetProperty] SetColor 0 0 0

# Create the usual rendering stuff
vtkRenderer renderer
 vtkRenderWindow renWin
 renWin AddRenderer renderer

vtkRenderWindowInteractor iren
 iren SetRenderWindow renWin

renderer AddActor sgridActor
renderer SetBackground 1 1 1
[renderer GetActiveCamera] Elevation 60.0
[renderer GetActiveCamera] Azimuth 30.0
[renderer GetActiveCamera] Zoom 1.25
renWin SetSize 300 300

# interact with data
 renWin Render
 iren Start

# Clean up and end
 vtkCommand DeleteAllObjects
 
puts end
exit