#
# CSCIE-236 Homework 2      
# Russell Lowke  March 22nd 2005
#
# Question 5.3  Render the image data using Ray-casting - 20pts
#
# Render volume of Monkey Brain using ray casting
# 
# This script is largely based off the vtk example SimpleRayCast.tcl
#
# I have defined two important transfer functions:
#
#   #1) is at 20.268, It colors the monkey brain RED over
#	   the same area/iso value I used in RenderMonkey_Q5_2.tcl 
#	   to render Marching cubes. 
#
#	#2) ranges to 180, it tints portions of the monkey skin BLUE
#	    translucently colorizing parts of the neck, shoulders, ears,
#		eyes and nose.
#		
#
#
# ... when convverting C++ to TCL
# anActor->GetProperty()->SetColor(red, green, blue);     is
# [anActor GetProperty] SetColor $red $green $blue

#
# First include the VTK Tcl packages which will make available 
# all of the vtk commands to Tcl
#
package require vtk
package require vtkinteraction
package require vtktesting

# Create the standard renderer, render window
# and interactor
vtkRenderer renderer
vtkRenderWindow renWin
 renWin AddRenderer renderer
vtkRenderWindowInteractor iren
 iren SetRenderWindow renWin

# Create the reader for the data
vtkStructuredPointsReader reader
 reader SetFileName "MonkeyBrain.vtk"


# Cast the data into unsigned shorts for ray casting
vtkImageCast cast
 cast SetInput [reader GetOutput]
 cast SetOutputScalarTypeToUnsignedShort
  
  
# Create transfer mapping scalar value to opacity
vtkPiecewiseFunction opacityTransferFunction
 opacityTransferFunction AddPoint  18   0.0
 opacityTransferFunction AddPoint  255  0.2

# Create transfer mapping scalar value to color
vtkColorTransferFunction colorTransferFunction
 colorTransferFunction AddRGBPoint     20.268 1.0 0.0 0.0
 colorTransferFunction AddRGBPoint    180.0 0.0 0.0 0.3

# The property describes how the data will look
vtkVolumeProperty volumeProperty
 volumeProperty SetColor colorTransferFunction
 volumeProperty SetScalarOpacity opacityTransferFunction
 volumeProperty ShadeOn
 volumeProperty SetInterpolationTypeToLinear

# The mapper / ray cast function know how to render the data
vtkVolumeRayCastCompositeFunction  compositeFunction
vtkVolumeRayCastMapper volumeMapper
 volumeMapper SetVolumeRayCastFunction compositeFunction
 volumeMapper SetInput [cast GetOutput]

# The volume holds the mapper and the property and
# can be used to position/orient the volume
vtkVolume volume
 volume SetMapper volumeMapper
 volume SetProperty volumeProperty

renderer AddVolume volume
renderer SetBackground 1 1 1
renWin SetSize 500 500
renWin Render

# turn the camera by $amount using $turns times
proc turn_cube { turns amount } {
	for {set i 0} {$i<$turns} {incr i} {
		[renderer GetActiveCamera] Azimuth $amount
		[renderer GetActiveCamera] Roll $amount
		renWin Render
		puts "Animating at frame [expr $i+1] of $turns"
	}
}

turn_cube 8 15

puts "end"