#
# CSCIE-236 Homework 2      
# Russell Lowke  March 22nd 2005
#
# Question 4.2  Render the data set with Marching Cubes - 25pts
#
# Read the Gaussian data previously created and render using Marching Cubes.
# Animate the iso-surfaces in a loop and pick a good color map.
#
#  Note: The iso surface values I have used to animate the Gaussian are taken from
#        evenly spaced samples of the data.  As such, the contours are evenly spaced.
#
#  Note: Scalar contouring is implemented in the Visualization Toolkit with vtkContourFilter,
#        and contouring in VTK is implemented using the marching cubes algorithm.
#		 In short, vtkContourFilter calls vtkMarchingCubes when it assertains that it is
#		 contouring a 3D volume.  See p178 of "The Visualization Toolkit, 3rd Ed."
# 
# ... when convverting C++ to TCL
# anActor->GetProperty()->SetColor(red, green, blue);     is
# [anActor GetProperty] SetColor $red $green $blue

#
# First include the VTK Tcl packages which will make available 
# all of the vtk commands to Tcl
#
package require vtk
package require vtkinteraction
package require vtktesting

puts "Reading Gaussian data from gaussianDataPts.vtk"
# read in the StructuredPoints data created by CreateGaussian(Q4.1).tcl
vtkStructuredPointsReader reader
 reader SetFileName "gaussianDataPts.vtk"
 reader Update

# number of contour increments to animate through
set inc 16
# color difference from first contour to last
set colDiff 0.5
# color difference between each contour
set colJmp [expr $colDiff/$inc]

# create a renderer, a render window, and a window interactor
vtkRenderer renderer
vtkRenderWindow renWin
 renWin AddRenderer renderer
vtkRenderWindowInteractor iren
 iren SetRenderWindow renWin

# create a series of iso contours to animate through, each with their own actor
puts "Creating contours..."
for {set i 1} {$i<=$inc} {incr i} {
  
	# find the contour value for this iteration
	switch $i {
		16 { set val 5.06398596999e-13 }
		15 { set val 5.03223951988e-12 }
		14 { set val 4.31211357615e-11 }
		13 { set val 3.18624491191e-10 }
		12 { set val 2.03015031807e-09 }
		11 { set val 1.11541713758e-08 }
		10 { set val 5.28453169363e-08 }
		9  { set val 2.15891616291e-07 }
		8  { set val 7.60545367371e-07 }
		7  { set val 2.3103328307e-06 }
		6  { set val 6.05179280573e-06 }
		5  { set val 1.36695314518e-05 }
		4  { set val 2.66246117337e-05 }
		3  { set val 4.47170365408e-05 }
		2  { set val 6.47623832919e-05 }
		1  { set val 8.08784291279e-05 }
		#0 { set val 8.70968942856e-05 }
	}
	
	puts "#$i contour = $val"
    
	# create iso-surface
	vtkContourFilter "contour$i"
	 "contour$i" SetInput [reader GetOutput]
	 "contour$i" SetValue 0 $val

	# create mapper 
	vtkPolyDataMapper "mapper$i"
	 "mapper$i" SetInput [contour$i GetOutput]
	 "mapper$i" ScalarVisibilityOff

	# create actor and colorize according to increment
	vtkActor "actor$i"
	 "actor$i" SetMapper mapper$i
	 set red [expr 0.1 + $colJmp*$i]
	 set green [expr 0.3 + $colJmp*$i]
	 set blue [expr 0.3 + $colJmp*$i]
	["actor$i" GetProperty] SetColor $red $green $blue
      
	renderer AddActor "actor$i"
} 

# render window parameters
renderer SetBackground 1 1 1
renWin SetSize 500 500

puts "Contours are evenly spaced."
puts "Please wait, calculating (large) animation using $inc contours..."
renWin Render
puts "ok..."

#
# my TCL processes used for animating the iso data lines

# turn the data $turns times
proc turn_cube { turns } {
	for {set i 0} {$i<$turns} {incr i} {
		[renderer GetActiveCamera] Azimuth 1
		[renderer GetActiveCamera] Roll 1
		renWin Render
	}
}

# add iso line
proc add_actor { n } { renderer AddActor "actor$n" }

# remove iso line
proc remove_actor { n } { renderer RemoveActor "actor$n" }

# add all the contour lines
proc add_contours { } {
	global inc
	for {set i 1} {$i<=$inc} {incr i} {
		add_actor $i
		turn_cube 3
	}
}

# remove all the contour lines
proc remove_contours { } {
	global inc
	for {set i 0} {$i<$inc} {incr i} {
		remove_actor [expr $inc - $i]
		turn_cube 3
	}
}

# animate all the contour lines out then in, $times
proc animate_contours { times } {
	global inc
	for {set j 0} {$j<$times} {incr j} {
	
		# animate iso lines out
		for {set i 1} {$i<=$inc} {incr i} {
			add_actor $i
			if {$i>1} { remove_actor [expr $i-1] }
			turn_cube 5
		}
		
		# animate iso lines in
		for {set i 1} {$i<$inc} {incr i} {
			add_actor [expr $inc - $i]
			remove_actor [expr $inc-$i+1]
			turn_cube 5
		}
		remove_actor 1
	}
}

# build all the contour lines then remove them $times
proc animate_contours_b { times } {
	for {set j 0} {$j<$times} {incr j} {
		add_contours
		turn_cube 64
		remove_contours
	}
}

# animation dance
remove_contours
animate_contours 3
animate_contours_b 2
animate_contours 3
add_contours
turn_cube 100

puts "Now you may interact with the data"
# interact with data
iren Start

# Clean up and end
vtkCommand DeleteAllObjects
puts end

exit


