#
# CSCIE-236 Homework 3  
# Russell Lowke  April 27th 2005
#

# REQUIRED CONSTANTS
set rake_start [split "3.27323 4.16378 2.86213"]
set rake_end   [split "-0.5 0.4 2.86213"]
set n_particles 6
set size_of_particles 0.1
set time_step 0.02
set simulation_length 0.6

package require vtk
package require vtkinteraction

vtkPLOT3DReader pl3d
	# pl3d SetXYZFileName "combxyz.bin"
	# pl3d SetQFileName "combq.bin"
    pl3d SetXYZFileName "bluntfinxyz.bin"
    pl3d SetQFileName "bluntfinq.bin"
#	pl3d SetScalarFunctionNumber 100
    pl3d SetVectorFunctionNumber 202
    pl3d Update
    
# create structured grid
vtkStructuredGrid sgrid
	sgrid DeepCopy [pl3d GetOutput]
	
# extract grid info
set nPoints [sgrid GetNumberOfPoints]
set dimensions [split [sgrid GetDimensions]]
set bounds [split [sgrid GetBounds]]
set center [split [sgrid GetCenter]]
set low_point [lindex $bounds 4]
set high_point [lindex $bounds 5]

puts "center: $center"
puts "bounds: $bounds"

#### Procedures

# scale a tuple by value
#
proc scale { val tpl } {
	
	set x [expr $val * [lindex $tpl 0]]
	set y [expr $val * [lindex $tpl 1]]
	set z [expr $val * [lindex $tpl 2]]
	
	return [split "$x $y $z"]
}

# divide a tuple by value
#
proc div { tpl val } {
	
	set x [expr [lindex $tpl 0] / $val]
	set y [expr [lindex $tpl 1] / $val]
	set z [expr [lindex $tpl 2] / $val]
	
	return [split "$x $y $z"]
}

# add two tuples together
#
proc add { tpl_a tpl_b } {

	set x [expr [lindex $tpl_a 0] + [lindex $tpl_b 0]]
	set y [expr [lindex $tpl_a 1] + [lindex $tpl_b 1]]
	set z [expr [lindex $tpl_a 2] + [lindex $tpl_b 2]]
	
	return [split "$x $y $z"]
}

# add four tuples together
#
proc add4 { tpl_a tpl_b tpl_c tpl_d } {

	set x [expr [lindex $tpl_a 0] + [lindex $tpl_b 0] + [lindex $tpl_c 0] + [lindex $tpl_d 0]]
	set y [expr [lindex $tpl_a 1] + [lindex $tpl_b 1] + [lindex $tpl_c 1] + [lindex $tpl_d 1]]
	set z [expr [lindex $tpl_a 2] + [lindex $tpl_b 2] + [lindex $tpl_c 2] + [lindex $tpl_d 2]]
	
	return [split "$x $y $z"]
}

proc vector_at { loc t } {
	global sgrid
	# NOTE: time variable t not used due to restriction in vtkPLOT3DReader
	#		which only reads steady flows
	#	set point [sgrid FindPoint $x $y $z $t]
	
	set x [lindex $loc 0]
	set y [lindex $loc 1]
	set z [lindex $loc 2]
	
	set point [sgrid FindPoint $x $y $z]
	set vector [split [sgrid GetPoint $point]]
	
	return $vector
}

# check if location at limit of bounds
#
proc check_limit { loc } {
	global bounds
	
	set x [lindex $loc 0]
	set y [lindex $loc 1]
	set z [lindex $loc 2]
	
	if {$x < [lindex $bounds 0] || $y < [lindex $bounds 2] || $z < [lindex $bounds 4] || $x > [lindex $bounds 1] || $y > [lindex $bounds 3] || $z > [lindex $bounds 5] } { 
		return 1
	} else {
		return 0
	}
}

# perform fourth order Runge-Kutta scheme
proc runge_kutta_4th { t delta_t loc } {
	
	# delta_t * v(p(t), t)						estimated change in position from start point		
	set a [scale $delta_t [vector_at $loc $t]]
	
	# delta_t * v(p(t) + a/2, t + delta_t/2)	estimated change half way along
	set b [scale $delta_t [vector_at [add $loc [div $a 2]] [expr $t + $delta_t/2]]]
	
	# delta_t * v(p(t) + b/2, t + delta_t/2)	estimated change half way along
	set c [scale $delta_t [vector_at [add $loc [div $b 2]] [expr $t + $delta_t/2]]]
	
	# delta_t * v(p(t) + c, t + delta_t)		estimate change in position from estimated end point
	set d [scale $delta_t [vector_at [add $loc $c] [expr $t + $delta_t]]]
	
	# p(t + delta_t) = p(t) + (a + 2b + 2c + d)/6
	return [add $loc [div [add4 $a [scale 2 $b] [scale 2 $c] $d] 6]]
	
}

#### 



# Create the usual graphics stuff/
#
vtkRenderer render
vtkRenderWindow renWin
    renWin AddRenderer render
vtkRenderWindowInteractor iren
    iren SetRenderWindow renWin
    
# The outline provides context for the data and the planes.
vtkStructuredGridOutlineFilter outline
    outline SetInput [pl3d GetOutput]
vtkPolyDataMapper outlineMapper
    outlineMapper SetInput [outline GetOutput]
vtkActor outlineActor
    outlineActor SetMapper outlineMapper
    [outlineActor GetProperty] SetColor 0 0 0
render AddActor outlineActor

# for curiosity, display the structured grid
vtkHedgeHog hedgehog
	hedgehog SetInput sgrid
	hedgehog SetScaleFactor 0.1
vtkPolyDataMapper sgridMapper
	sgridMapper SetInput [hedgehog GetOutput]
vtkActor sgridActor
	sgridActor SetMapper sgridMapper
	[sgridActor GetProperty] SetColor 0 0 0
# render AddActor sgridActor


set increments [expr $simulation_length / $time_step]

puts "rake_start: $rake_start"
puts "rake_end: $rake_end"
puts "increments: $increments"

# find vars for calculating rake
set cx [expr ([lindex $rake_end 0] - [lindex $rake_start 0]) / ($n_particles - 1)]
set cy [expr ([lindex $rake_end 1] - [lindex $rake_start 1]) / ($n_particles - 1)]
set cz [expr ([lindex $rake_end 2] - [lindex $rake_start 2]) / ($n_particles - 1)]
	
# create particles
set n 0
for {set i 0} {$i <= $increments} {incr i} {
	for {set j 0} {$j < $n_particles} {incr j} {
	
		set x [expr [lindex $rake_start 0] + $cx * $j]
		set y [expr [lindex $rake_start 1] + $cy * $j]
		set z [expr [lindex $rake_start 2] + $cz * $j]

		set "sphLoc$n" [split "$x $y $z"]
		eval "set temp \$sphLoc$n"
		
		if {$i == 1} {
			puts "rake point $j at: $temp"
		}
	
		vtkSphereSource "sphere$n"
	    	"sphere$n" SetPhiResolution 12
    		"sphere$n" SetThetaResolution 12
			"sphere$n" SetRadius $size_of_particles
			"sphere$n" SetCenter [lindex $temp 0] [lindex $temp 1] [lindex $temp 2]
		vtkElevationFilter "colorIt$n"
			"colorIt$n" SetInput ["sphere$n" GetOutput]
			"colorIt$n" SetLowPoint 0 0 $low_point
			"colorIt$n" SetHighPoint 0 0 $high_point
		vtkPolyDataMapper "mapper$n"
			"mapper$n" SetInput ["colorIt$n" GetPolyDataOutput] 
			eval "mapper$n" SetScalarRange [[pl3d GetOutput] GetScalarRange]
		vtkActor "sphereActor$n"
			"sphereActor$n" SetMapper "mapper$n"
		render AddActor "sphereActor$n"
		
		incr n
	}
}

render SetBackground 1 1 1
renWin SetSize 500 500

# Create an initial view
set cam1 [render GetActiveCamera]
iren Initialize

  
# tip camera on angle
[render GetActiveCamera] Azimuth -160.45
[render GetActiveCamera] Roll 31.2595
[render GetActiveCamera] Elevation 30

set t 0
set old_t 0
set n $n_particles
for {set i 0} {$i < $increments} {incr i} {
	
	# find $delta_t, should be the same as $time_step
	set delta_t [expr $t - $old_t]
	
	puts "$i) t = $t		delta_t = $delta_t"
	
	# update particles
	for {set j 0} {$j < $n} {incr j} {
		eval "set oLoc \$sphLoc$j"
		set nLoc [runge_kutta_4th $t $delta_t $oLoc];
		if {[check_limit $nLoc] == 1} {
			set "sphLoc$j" $oLoc
		} else {
			set "sphLoc$j" $nLoc
		}
		eval "set temp \$sphLoc$j"
		"sphere$j" SetCenter [lindex $temp 0] [lindex $temp 1] [lindex $temp 2]
	}
	
	renWin Render
	
	# update time values
	set old_t $t
	set t [expr $t + $time_step]
	
	# export as jpeg
	vtkWindowToImageFilter w2i
		w2i SetInput renWin
	vtkJPEGWriter writer
		writer SetInput [w2i GetOutput]
		writer SetFileName "plot_bluntfin_images/plot_bluntfin_$i.jpg"
		writer Write
	
	# clear the capture objects
	w2i Delete
	writer Delete
	
	# add new particles to rake
	set n [expr $n + $n_particles]
}


# render the image
#
iren AddObserver UserEvent {wm deiconify .vtkInteract}
renWin Render

# prevent the tk window from showing up then start the event loop
wm withdraw .



